#include "player.h"
#include "debuggers.h"
#include "olcPGEX_Sound.h"
#include <iostream>

namespace pabloader {
Player::Player(Debuggers* game_)
    : Entity(game_)
{
    w = game->playerSprite.width / 4;
    h = game->playerSprite.height / 4;
    x = (game->ScreenWidth() - w) / 2;
    y = game->GameScreenHeight() / 2;
}

void Player::Update(float dt)
{
    x += xv * dt;
    y += yv * dt;

    xv *= 1 - 0.2 * dt;

    if (!onGround) {
        yv += (70 + skin * 20) * dt;
        yv *= 1 - 0.05 * dt;
    } else {
        xv *= 1 - 0.5 * dt;
    }

    transform.Reset();

#ifdef _DEBUG
    // game->DrawString(1, 19, std::to_string(x) + " " + std::to_string(y));
    // game->DrawString(1, 28, std::to_string(xv) + " " + std::to_string(yv));

    if (game->GetKey(olc::J).bPressed) {
        skin = (skin + 1) % 4;
    }
#endif
    bool userInput = false;

    if (game->GetKey(olc::SPACE).bHeld && onGround) {
        yv = -100;
        userInput = true;
    }

    if (game->GetKey(olc::A).bHeld) {
        xv -= (100 + skin * 50) * dt;
        userInput = true;
    }

    if (game->GetKey(olc::D).bHeld) {
        xv += (100 + skin * 50) * dt;
        userInput = true;
    }

    if (game->GetKey(olc::S).bHeld) {
        if (!onGround) {
            yv += (100 + skin * 50) * dt;
        }
        userInput = true;
    }

    if (x > game->ScreenWidth() - w / 2) {
        xv *= -0.3;
        x = game->ScreenWidth() - w / 2;
        olc::SOUND::PlaySample(game->playerFallSample);
    }

    if (x < w / 2) {
        xv *= -0.3;
        x = w / 2;
        olc::SOUND::PlaySample(game->playerFallSample);
    }

    if (y > game->GameScreenHeight() - h / 2) {
        yv *= -0.3;

        if (std::abs(yv) < 2) {
            yv = 0;
        }
        y = game->GameScreenHeight() - h / 2;
    }

    bool newOnGround = y >= game->GameScreenHeight() - h / 2;
    if (newOnGround && !onGround) {
        olc::SOUND::PlaySample(game->playerFallSample);
    }
    onGround = newOnGround;

    if (onGround) {
        tile = 2;
        tileCounter = static_cast<uint32_t>(0.1 / dt);
    } else if (tileCounter <= 0) {
        tile = 0;
    } else {
        tileCounter--;
    }

    if (!userInput && std::abs(xv) < 2) {
        xv = 0;
    }
    if (xv < 0) {
        transform.Scale(-1, 1);
    }
    transform.Translate(x, y);
}

void Player::Draw()
{
    int xOff = xv < 0 ? -1 : 0;
    if (std::abs(xv) < 1) {
        olc::GFX2D::DrawPartialSprite(&game->playerSprite, skin * w + xOff, h + tile * h, w, h, transform, true);
    } else {
        olc::GFX2D::DrawPartialSprite(&game->playerSprite, skin * w + xOff, 0 + tile * h, w, h, transform, true);
    }
#ifdef _DEBUG
    float x, y;
    transform.Forward(0, 0, x, y);
    game->DrawRect((int)x - w / 2, (int)y - h / 2, w, h, olc::RED);
#endif
}
}